// Copyright 2015, Jean Rene Dawin
//
// This Library is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 2.1 of the License, or
// (at your option) any later version.
//
// This Library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this Library.  If not, see <http://www.gnu.org/licenses/>.

#include "escwyp_misch.h"
#include "escwyp_out_txt_extern.h"
#include "escwyp_oszillo_extern.h"
#include "escwyp_rec_gui_elm_extern.h"
#include "escwyp_main.h"
#include <stdio.h>

void misch_init()
{
   fprintf(stdout, "Extrablatt ************* klavier_max_freq_array_size = %i\n", klavier_max_freq_array_size);
   int i;
   for(i = 0; i < MAX_NUM_NOTES; ++i){
      nts[i] = 0;
   }
   for(i = 0; i < klavier_max_freq_array_size; ++i){
      ixs[i] = -1;
   }
}
void misch_init_buffer(const int length)
{
   if(mix_buff){
      free(mix_buff);
   }
   mix_buff = (float *) malloc(length*sizeof(float));
}

void misch(int length){
   //fprintf(stdout, "Deuteruim\n");
   int i, j;
   int overwrite_mix_buff = 0;
   for(i = 0; i < MAX_NUM_NOTES && num_nts > 0; ++i){
      if(nts[i] != 0){
         if(overwrite_mix_buff == 0){
            overwrite_mix_buff = 1;
         }
         //fprintf(stdout, "Natruim %i\n", nts[i]);
         set_params(current_sound_function, i);
         refresh_params(current_sound_function); 
         call_sound_func(current_sound_function, length);
         if(overwrite_mix_buff == 1){
            for(j = 0; j < length; ++j){
               mix_buff[j] = buff_L[j];
            }
            overwrite_mix_buff = 2;
         }else{
            for(j = 0; j < length; ++j){
               mix_buff[j] += buff_L[j];
            }
         }
      }
   }
   for(i = 0; i < length; ++i){
      mix_buff[i] *= mix_volume;
   }
   memcpy(buff_L, mix_buff, length*sizeof(float));
   oszilloCopyBuffer(length, mix_buff);
   //if(num_nts > 0){
   //   output_txt = 1;
   //}else{
   //   output_txt = 0;
   //}
   //outTxtCopy(length, mix_buff);
   //bar_rec_fun(0.0);
   //fprintf(stdout, "J\n");
}

void call_sound_func(int id, int length){

   switch(id){

      case 1: { makeChains(length, buff_L, samp_rate_jack); break; }
      case 2: { makeChains(length, buff_L, samp_rate_jack); break; }
      case 3: { makeSineRobin(length, buff_L, samp_rate_jack); break; }
      case 4: { makeSineGrass(length, buff_L); break; }
      case 5: { makeNadelper(length, buff_L); break; }
      case 6: { makeSamplecast(length, buff_L); break; }
      case 7: { makeSpratz(length, buff_L); break; }

   }
}

void set_params(int id, int stimme){

   switch(id){

      case 1: { chain_set_params(&the_ch_params[stimme]); break; }
      case 2: { chain_set_params(&the_ch_params[stimme]); break; }
      case 3: { sine_robin_set_params(&the_sr_params[stimme]); break; }
      case 4: { sine_grass_set_params(&the_sg_params[stimme]); break; }
      case 5: { nadelper_set_params(&the_np_params[stimme]); break; }
      case 6: { samplecast_set_params(&the_sc_params[stimme]); break; }
      case 7: { spratz_set_params(&the_sp_params[stimme]); break; }

   }
}

void refresh_params(int id){

   switch(id){

      case 1: { chain_refresh_parameters(); break; }
      case 2: { chain_refresh_parameters(); break; }
      case 3: { sine_robin_refresh_parameters(); break; }
      case 4: { sine_grass_refresh_parameters(); break; }
      case 5: { nadelper_refresh_parameters(); break; }
      case 6: { samplecast_refresh_parameters(); break; }
      case 7: { spratz_refresh_parameters(); break; }

   }
}

void initialise_params(const int sound_method, const int note_id){

   switch(sound_method){

      case 1: { 
                 chain_set_params(&the_ch_params[ixs[note_id]]);
                 //chain_init_values();
                 //chain_set_freq_from_klavier(freqs_by_index[note_id], freqs_by_index[0], freqs_by_index[131]); 
                 chain_set_freq_from_klavier((float) note_id, (float) 12, (float) 131); 
                 break;
              }

      case 2: { 
                 chain_set_params(&the_ch_params[ixs[note_id]]);
                 //chain_init_values();
                 chain_set_freq_from_klavier(freqs_by_index[note_id], freqs_by_index[0], freqs_by_index[131]); 
                 break;
              }

      case 3: { 
                 sine_robin_set_params(&the_sr_params[ixs[note_id]]);
                 sine_robin_init_values_form_klavier((float) note_id, (float) 12, (float) 131); 
                 //sine_robin_init_values(freqs_by_index[note_id]/klavierGetMaxFreq());
                 break;
              }

      case 4: { 
                 sine_grass_set_params(&the_sg_params[ixs[note_id]]);
                 sine_grass_init_values(freqs_by_index[note_id], samp_rate_jack);
                 break;
              }

      case 5: {
                 nadelper_set_params(&the_np_params[ixs[note_id]]);
                 //nadelper_set_freq(klavierGetMaxFreq()-freqs_by_index[note_id]); 
		         nadelper_set_freq_from_klavier((float) note_id, (float) 12, (float) 131);
                 nadelper_init_values(); 
                 break;
              }
      case 6: {
                 samplecast_set_params(&the_sc_params[ixs[note_id]]);
                 samplecast_init_values(); 
                 samplecast_set_freq_from_klavier(freqs_by_index[note_id], freqs_by_index[0], freqs_by_index[131]);
                 break;
              }
      case 7: {
                 spratz_set_params(&the_sp_params[ixs[note_id]]);
                 spratz_init_values(); 
                 spratz_set_freq_from_klavier(freqs_by_index[note_id], klavierGetMinFreq(), klavierGetMaxFreq()); 
                 fprintf(stdout, "SPRATZ_FREQ = %f\n", freqs_by_index[note_id]);
                 break;
              }

   }
}

int next_hole(){

   if(num_hls > 0){
      return hls[--num_hls];
   }else{
      if(num_nts >= MAX_NUM_NOTES){
         return -1;
      }else{
         return num_nts;
      }
   }
}

void insert_hole(const int place){

   hls[num_hls++] = place;

}

int note_on(const int id){
   if(id >= 0 && id < klavier_max_freq_array_size){
      int tmp_ixs_id = ixs[id];
      if(tmp_ixs_id == -1){
         int new_index = next_hole();
         if(new_index < 0){
            fprintf(stdout, "Pardon, no space left.\n");
         }else{
            ixs[id] = new_index;
            nts[new_index] = id;
            ++num_nts;
            if(num_nts == 1){
                change_play_status(get_current_sound_func());
                outTxtNotes(); //Call this to notify out_txt that notes just startet playing
            }
            initialise_params(current_sound_function, id);
            //sine_grass_set_freq(id*40.0);
            print_note_params();
            //print_params_sine_grass(&the_sg_params[new_index]);
            print_nts();
            print_ixs();
            return 0;
         }
      }
   }
   return -1;
}

void escwyp_migrate_notes()
{
   int i;
   for(i = 0; i < MAX_NUM_NOTES && num_nts > 0; ++i){
      if(nts[i] > 0){
         initialise_params(current_sound_function, nts[i]);
      }
   }
   if(num_nts == 0){
      playsth = 0;
   }else{
      playsth = current_sound_function;
   }
}

int note_off(const int id){
   if(id >= 0 && id < klavier_max_freq_array_size){
      int tmp_ixs_id = ixs[id];
      if(tmp_ixs_id >= 0){
          print_note_params();
         nts[tmp_ixs_id] = 0;
         insert_hole(tmp_ixs_id);
         ixs[id] = -1;
         --num_nts;
         if(num_nts == 0){
             change_play_status(-1*get_current_sound_func());
             outTxtSilence(); //Call this to notify out_txt that notes have ceased and we have silence
         }
         print_nts();
         print_ixs();
         return 0;
      }else{
         fprintf(stdout, "Pardon, cannot off this note id.\n");
      }
   }
   return -1;
}

void clear_notes(){

   int i;
   fprintf(stdout, "CLEARING NTS\n");
   for(i = 0; i < MAX_NUM_NOTES; ++i){
      if(nts[i] > 0){
         ixs[nts[i]] = -1;
         nts[i] = 0;
      }
   }
   num_nts = 0;
   num_hls = 0;
   fprintf(stdout, "\n");

}
void print_nts(){

   int i;
   fprintf(stdout, "NTS\n");
   for(i = 0; i < MAX_NUM_NOTES; ++i){
      if(nts[i] > 0){
         fprintf(stdout, "%i = %i\n", i, nts[i]);
      }
   }
   fprintf(stdout, "\n");
}

void print_ixs(){

   int i;
   fprintf(stdout, "IXS\n");
   for(i = 0; i < klavier_max_freq_array_size; ++i){
      if(ixs[i] != -1){
         fprintf(stdout, "%i = %i\n", i, ixs[i]);
      }
   }
   fprintf(stdout, "\n");
}

void print_note_params(){

   int i;
   fprintf(stdout, "NT-params\n");
   for(i = 0; i < MAX_NUM_NOTES; ++i){
      if(nts[i] > 0){
         fprintf(stdout, "............\n");
         fprintf(stdout, "%i[%i]\n", i, nts[i]);
         switch(current_sound_function){
            case 3: { print_params_sine_robin(&the_sr_params[i]); break; }
            case 4: { print_params_sine_grass(&the_sg_params[i]); break; }
            case 5: { print_params_nadelper(&the_np_params[i]); break; }
            case 6: { print_params_samplecast(&the_sc_params[i]); break; }
         } 
      }
   }
   fprintf(stdout, "\n");

}

int get_current_sound_func(){

   return current_sound_function;

}

void set_mix_volume(const float a_volume){

   if(a_volume >= 0.0 && a_volume <= 10.0){
      mix_volume = a_volume;
   }else{
      fprintf(stdout, "bad try to set mix_volume (%f)\n", a_volume);
   }

}

float get_mix_volume(){

   return mix_volume;

}

int get_playing_status(){

    int playing = 1;
    if(num_nts == 0){
        playing = 0;
    }

    return playing;
}
void misch_save_num_nts()
{

   num_nts_saved = num_nts;
   num_nts = 0;

}
void misch_restore_num_nts()
{

   num_nts = num_nts_saved ;

}

int note_is_set(const int id)
{

   if(id >= 0 && id < klavier_max_freq_array_size){
      if(ixs[id] >= 0){
         return 0;
      }
   }
   return -1;

}
