// Copyright 2015, Jean Rene Dawin
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version. See License-GPL-2.txt.

#include "escwyp_nadelper_gui_elm.h"
#include "escwyp_nadelper_extern.h"
#include "escwyp_globals_extern.h"
#include <Elementary.h>
#include "random_aux.h"

void nadelperPackGui(Evas_Object *box){

   elm_box_pack_end(box, nadelper_nadel_size_slider);
   evas_object_show(nadelper_nadel_size_slider);

   elm_box_pack_end(box, nadelper_num_freqs_slider);
   evas_object_show(nadelper_num_freqs_slider);

   elm_box_pack_end(box, nadelper_lowest_freq_slider);
   evas_object_show(nadelper_lowest_freq_slider);
   if(nadelper_get_self_control()){
      elm_object_disabled_set(nadelper_lowest_freq_slider, EINA_FALSE);
   }else{
      elm_object_disabled_set(nadelper_lowest_freq_slider, EINA_TRUE);
   }

   elm_box_pack_end(box, nadelper_highest_freq_slider);
   evas_object_show(nadelper_highest_freq_slider);

   elm_box_pack_end(box, nadelper_freq_step_size_slider);
   evas_object_show(nadelper_freq_step_size_slider);

   elm_box_pack_end(box, nadelper_random_step_size_active_check);
   evas_object_show(nadelper_random_step_size_active_check);

   elm_box_pack_end(box, nadelper_step_size_random_range_slider);
   evas_object_show(nadelper_step_size_random_range_slider);

}

void nadelperUnpackGui(Evas_Object *box){

   elm_box_unpack(box, nadelper_nadel_size_slider);
   evas_object_hide(nadelper_nadel_size_slider);

   elm_box_unpack(box, nadelper_num_freqs_slider);
   evas_object_hide(nadelper_num_freqs_slider);

   elm_box_unpack(box, nadelper_lowest_freq_slider);
   evas_object_hide(nadelper_lowest_freq_slider);

   elm_box_unpack(box, nadelper_highest_freq_slider);
   evas_object_hide(nadelper_highest_freq_slider);

   elm_box_unpack(box, nadelper_freq_step_size_slider);
   evas_object_hide(nadelper_freq_step_size_slider);

   elm_box_unpack(box, nadelper_random_step_size_active_check);
   evas_object_hide(nadelper_random_step_size_active_check);

   elm_box_unpack(box, nadelper_step_size_random_range_slider);
   evas_object_hide(nadelper_step_size_random_range_slider);

}

void nadelperInitGui(int hidden){

   //nadelper_default_params(the_params);

   // the slider setting the Verstaerkung/size of the rsulting Nadels
   nadelper_nadel_size_slider = elm_slider_add(win);
   evas_object_size_hint_align_set(nadelper_nadel_size_slider, EVAS_HINT_FILL, 0.5);
   evas_object_size_hint_weight_set(nadelper_nadel_size_slider, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   elm_slider_unit_format_set(nadelper_nadel_size_slider, "%1.3f VERSTAERK.");
   elm_slider_indicator_show_set(nadelper_nadel_size_slider, EINA_FALSE);
   elm_slider_min_max_set(nadelper_nadel_size_slider, 0.001, 0.5);
   elm_slider_value_set(nadelper_nadel_size_slider, nadelper_get_nadel_size());
   elm_slider_span_size_set(nadelper_nadel_size_slider, 200);
   evas_object_smart_callback_add(nadelper_nadel_size_slider, "delay,changed", nadelper_nadel_size_slider_fun, NULL);    

   // the slider setting the number of frequencies
   nadelper_num_freqs_slider = elm_slider_add(win);
   evas_object_size_hint_align_set(nadelper_num_freqs_slider, EVAS_HINT_FILL, 0.5);
   evas_object_size_hint_weight_set(nadelper_num_freqs_slider, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   elm_slider_unit_format_set(nadelper_num_freqs_slider, "%1.0f NUM_FREQS");
   elm_slider_indicator_show_set(nadelper_num_freqs_slider, EINA_FALSE);
   elm_slider_min_max_set(nadelper_num_freqs_slider, 0.0, nadelper_get_max_num_freqs());
   elm_slider_value_set(nadelper_num_freqs_slider, nadelper_get_num_freqs());
   elm_slider_span_size_set(nadelper_num_freqs_slider, 200);
   evas_object_smart_callback_add(nadelper_num_freqs_slider, "delay,changed", nadelper_num_freqs_slider_fun, NULL);    

   // the slider setting the lowest frequency or start-frequency
   nadelper_lowest_freq_slider = elm_slider_add(win);
   evas_object_size_hint_align_set(nadelper_lowest_freq_slider, EVAS_HINT_FILL, 0.5);
   evas_object_size_hint_weight_set(nadelper_lowest_freq_slider, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   elm_slider_unit_format_set(nadelper_lowest_freq_slider, "%1.0f LOWEST_FREQ");
   elm_slider_indicator_show_set(nadelper_lowest_freq_slider, EINA_FALSE);
   elm_slider_min_max_set(nadelper_lowest_freq_slider, 1.0, 1000.0);
   elm_slider_value_set(nadelper_lowest_freq_slider, nadelper_params_lowest_freq_get());
   elm_slider_span_size_set(nadelper_lowest_freq_slider, 200);
   evas_object_smart_callback_add(nadelper_lowest_freq_slider, "delay,changed", nadelper_lowest_freq_slider_fun, NULL);    

   // the slider setting the highest reachable frequency
   nadelper_highest_freq_slider = elm_slider_add(win);
   evas_object_size_hint_align_set(nadelper_highest_freq_slider, EVAS_HINT_FILL, 0.5);
   evas_object_size_hint_weight_set(nadelper_highest_freq_slider, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   elm_slider_unit_format_set(nadelper_highest_freq_slider, "%1.0f HIGHEST_FREQ");
   elm_slider_indicator_show_set(nadelper_highest_freq_slider, EINA_FALSE);
   elm_slider_min_max_set(nadelper_highest_freq_slider, 1.0, 20000.0);
   elm_slider_value_set(nadelper_highest_freq_slider, nadelper_get_highest_freq());
   elm_slider_span_size_set(nadelper_highest_freq_slider, 200);
   evas_object_smart_callback_add(nadelper_highest_freq_slider, "delay,changed", nadelper_highest_freq_slider_fun, NULL);    

   // the slider setting the interval or step size used to generate the consecutiveFrequencies 
   //starting from lowest_freq up to highest_freq but at most num_freqs Frequencies in total
   nadelper_freq_step_size_slider = elm_slider_add(win);
   evas_object_size_hint_align_set(nadelper_freq_step_size_slider, EVAS_HINT_FILL, 0.5);
   evas_object_size_hint_weight_set(nadelper_freq_step_size_slider, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   elm_slider_unit_format_set(nadelper_freq_step_size_slider, "%1.0f FREQ_STEP_SIZE");
   elm_slider_indicator_show_set(nadelper_freq_step_size_slider, EINA_FALSE);
   elm_slider_min_max_set(nadelper_freq_step_size_slider, 1.0, 100.0);
   elm_slider_value_set(nadelper_freq_step_size_slider, nadelper_get_freq_step_size());
   elm_slider_span_size_set(nadelper_freq_step_size_slider, 200);
   evas_object_smart_callback_add(nadelper_freq_step_size_slider, "delay,changed", nadelper_freq_step_size_slider_fun, NULL);    

   // the ckeck-box deciding whether the slider nadelper_step_size_random_range_slider
   // has an effect (1) or not (0).
   nadelper_random_step_size_active_check =  elm_check_add(win);
   elm_check_state_set(nadelper_random_step_size_active_check, EINA_FALSE);
   elm_object_part_text_set(nadelper_random_step_size_active_check, NULL, "RANDOM_STEP_SIZE!");
   evas_object_smart_callback_add(nadelper_random_step_size_active_check, "changed", nadelper_random_step_size_active_check_fun, NULL);    

// the slider setting the range for randomising th step_size. Runs from 1 to (freq_step_size - 1). The effective step_size comes from the range [step_size-random_range, step_size+random_range]_random
   nadelper_step_size_random_range_slider = elm_slider_add(win);
   evas_object_size_hint_align_set(nadelper_step_size_random_range_slider, EVAS_HINT_FILL, 0.5);
   evas_object_size_hint_weight_set(nadelper_step_size_random_range_slider, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   elm_slider_unit_format_set(nadelper_step_size_random_range_slider, "%1.0f RANDOM_RANGE");
   elm_slider_indicator_show_set(nadelper_step_size_random_range_slider, EINA_FALSE);
   elm_slider_min_max_set(nadelper_step_size_random_range_slider, 1.0, nadelper_get_freq_step_size());
   elm_slider_value_set(nadelper_step_size_random_range_slider, nadelper_get_step_size_random_range());
   elm_slider_span_size_set(nadelper_step_size_random_range_slider, 200);
   evas_object_smart_callback_add(nadelper_step_size_random_range_slider, "delay,changed", nadelper_step_size_random_range_slider_fun, NULL);    

}

static void nadelper_nadel_size_slider_fun(void *data, Evas_Object *obj, void *event_info){

   fprintf(stdout, "Yis nadelper_nadel_size_slider_fun\n");
   float val = (float) elm_slider_value_get(obj);
   nadelper_set_nadel_size(val);
   fprintf(stdout, "NADEL_SIZE %f\n", nadelper_get_nadel_size());

}

static void nadelper_num_freqs_slider_fun(void *data, Evas_Object *obj, void *event_info){

   fprintf(stdout, "Yis nadelper_num_freqs_slider_fun\n");
   int val = (int) elm_slider_value_get(obj);
   nadelper_set_num_freqs(val);
   nadelper_init_values();
   fprintf(stdout, "NUM_FREQS %i\n", nadelper_get_num_freqs());

}

static void nadelper_lowest_freq_slider_fun(void *data, Evas_Object *obj, void *event_info){

   fprintf(stdout, "Yis nadelper_lowest_freq_slider_fun\n");
   int val = (int) elm_slider_value_get(obj);
   nadelper_params_lowest_freq_set(val);
   nadelper_init_values();
   fprintf(stdout, "LOWEST_FREQ %i\n", nadelper_params_lowest_freq_get());

}

static void nadelper_highest_freq_slider_fun(void *data, Evas_Object *obj, void *event_info){

   fprintf(stdout, "Yis nadelper_highest_freq_slider_fun\n");
   int val = (int) elm_slider_value_get(obj);
   nadelper_set_highest_freq(val);
   nadelper_init_values();
   fprintf(stdout, "HIGHEST_FREQ %i\n", nadelper_get_highest_freq());

}

static void nadelper_freq_step_size_slider_fun(void *data, Evas_Object *obj, void *event_info){

   fprintf(stdout, "Yis nadelper_freq_step_size_slider_fun\n");
   int val = (int) elm_slider_value_get(obj);
   nadelper_set_freq_step_size(val);
   elm_slider_min_max_set(nadelper_step_size_random_range_slider, 1.0, nadelper_get_freq_step_size());
   val = (int) elm_slider_value_get(nadelper_step_size_random_range_slider);
   nadelper_set_step_size_random_range(val);
   nadelper_init_values();
   fprintf(stdout, "STEP_SIZE %i, RANDOM_RANGE %i\n", nadelper_get_freq_step_size(), nadelper_get_step_size_random_range());

}

static void nadelper_random_step_size_active_check_fun(void *data, Evas_Object *obj, void *event_info){

   fprintf(stdout, "Yis nadelper_random_step_size_active_check_fun\n");
   int val = elm_check_state_get(obj);
   nadelper_set_random_step_size_active(val);
   nadelper_init_values();

}

static void nadelper_step_size_random_range_slider_fun(void *data, Evas_Object *obj, void *event_info){

   fprintf(stdout, "Yis nadelper_step_size_random_range_slider_fun\n");
   int val = (int) elm_slider_value_get(obj);
   nadelper_set_step_size_random_range(val);
   random_seed_update(0);
   nadelper_init_values();
   fprintf(stdout, "RANDOM_RANGE %i\n", nadelper_get_step_size_random_range());

}

// GET / SET ---

void nadelper_nadel_size_slider_set(const float value){

        elm_slider_value_set(nadelper_nadel_size_slider, (double) value);

}
void nadelper_num_freqs_slider_set(const int value){

        elm_slider_value_set(nadelper_num_freqs_slider, (double) value);

}
void nadelper_lowest_freq_slider_set(const int value){

        elm_slider_value_set(nadelper_lowest_freq_slider, (double) value);

}
void nadelper_highest_freq_slider_set(const int value){

        elm_slider_value_set(nadelper_highest_freq_slider, (double) value);

}
void nadelper_freq_step_size_slider_set(const int value){

        elm_slider_value_set(nadelper_freq_step_size_slider, (double) value);

}
void nadelper_random_step_size_active_check_set(const int value){

        if(value == 0){
                elm_check_state_set(nadelper_random_step_size_active_check, EINA_FALSE);
        }else{
                elm_check_state_set(nadelper_random_step_size_active_check, EINA_TRUE);
        }

}
void nadelper_step_size_random_range_slider_set(const int value){

        elm_slider_value_set(nadelper_step_size_random_range_slider, (double) value);

}

void nadelper_gui_set_self_control(const int value)
{
    nadelper_set_self_control(value);
    if(nadelper_get_self_control()){
       elm_object_disabled_set(nadelper_lowest_freq_slider, EINA_FALSE);
    }else{
       elm_object_disabled_set(nadelper_lowest_freq_slider, EINA_TRUE);
    }

}

