// Copyright 2014-2015, Jean Rene Dawin
//
// This Library is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 2.1 of the License, or
// (at your option) any later version.
//
// This Library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this Library.  If not, see <http://www.gnu.org/licenses/>.

//#include <Elementary.h>
//#include <Evas.h>
//#include <Imlib2.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "escwyp_sine_grass.h"



void sine_grass_default_params(struct sine_grass_params *the_params)
{

   the_params->sine_grass_freq = 100.8;
   the_params->grass_sign_count = 0;
   the_params->grass_sign = 1;
   the_params->lawn_length = 1;
   the_params->lawn_hill_index = 0; 
   the_params->grass_rnd = 0.0;
   the_params->lawn_length_variation_max = 0;
   the_params->lawn_length_variation = 0;

}

void print_params_sine_grass(struct sine_grass_params *the_params)
{

   if(the_params != NULL){

      fprintf(stdout, "-----------------\n");
      fprintf(stdout, "sine_grass_freq %f\n", the_params->sine_grass_freq);
      fprintf(stdout, "grass_sign_count %i\n", the_params->grass_sign_count);
      fprintf(stdout, "grass_sign %i\n", the_params->grass_sign);
      fprintf(stdout, "lawn_length %i\n", the_params->lawn_length);
      fprintf(stdout, "lawn_hill_index %i\n", the_params->lawn_hill_index); 
      fprintf(stdout, "grass_rnd %f\n", the_params->grass_rnd);
      fprintf(stdout, "lawn_length_variation_max %i\n", the_params->lawn_length_variation_max);
      fprintf(stdout, "lawn_length_variation %i\n", the_params->lawn_length_variation);
      fprintf(stdout, "-----------------\n");
   }else{
      fprintf(stdout, "Pardon, no params set.\n");
   }
}


void sine_grass_refresh_parameters()
{

   the_params->lawn_length = (int) trunc(sine_grass_srate/(2*the_params->sine_grass_freq));
   the_params->lawn_length_variation_max = (int) trunc(the_params->lawn_length*lawn_length_variation_frac); 
   if(the_params->lawn_length_variation_max  == 0){
      the_params->lawn_length_variation = 0;
   }

}

void sine_grass_set_params(struct sine_grass_params *params)
{

      the_params = params;
}
  
void sine_grass_set_freq(const float freq)
{
   
   the_params->sine_grass_freq = freq;

}

void sine_grass_init_values(const float freq, const unsigned int srate)
{

   sine_grass_srate = srate;
   if(freq <= 0.0){
      the_params->sine_grass_freq = 100.8;
   }else{
      if(!self_control){
	   the_params->sine_grass_freq = freq;
      }
   }

   the_params->lawn_length_variation = 0;

   // these are the length of one piece of lawn or one hill, and the index of the middle of the hill or lawn
   the_params->lawn_length = (int) trunc(srate/(2*the_params->sine_grass_freq));
   the_params->lawn_length_variation_max = (int) trunc(the_params->lawn_length*lawn_length_variation_frac); 
   the_params->lawn_hill_index = the_params->lawn_length/2;
   // the kind of period value for makeSineGrass. Remebers the number of samples completed.
   // Is checked in makeSineGrass to alter sign and value of the calculated samples.
   the_params->grass_sign_count = the_params->lawn_length;
   // the sign of the samples
   the_params->grass_sign = 1;
   // the random-value for the hill-top
   the_params->grass_rnd = (float) random()/RAND_MAX;

}


 //---------------- This function is called when key4 is pressed -------------------

void makeSineGrass(int length, float *buff_L)
{

   int k;
   int lawn_length_variation_sign = 1;
   float tmp_rnd = 0.0;
   for(k = 0; k < length; k++){

      if(the_params->grass_sign_count == 0){
         the_params->grass_sign *= -1;
         if(the_params->lawn_length_variation_max > 0){
            tmp_rnd = (float) random()/RAND_MAX;
            if(tmp_rnd < 0.5){
               lawn_length_variation_sign = 1;
            }else{
               lawn_length_variation_sign = -1;
            }
            the_params->lawn_length_variation = (int) lawn_length_variation_sign * trunc(the_params->lawn_length_variation_max * tmp_rnd);
         }
         the_params->grass_sign_count = the_params->lawn_length + the_params->lawn_length_variation;
         the_params->lawn_hill_index = the_params->grass_sign_count/2;
         the_params->grass_rnd = (float) random()/RAND_MAX;
      }
      if(the_params->grass_sign_count == the_params->lawn_hill_index || grass_mowed == 1){
         buff_L[k] = (float) the_params->grass_sign*the_params->grass_rnd;
      }else{
         buff_L[k] = (float) grass_hill_fraction*the_params->grass_sign*the_params->grass_rnd*random()/RAND_MAX;
      }
      the_params->grass_sign_count--;
   }
}

//-------------------------------------------------------------

int sine_grass_get_grass_mowed()
{

        return grass_mowed;

}

float sine_grass_get_grass_hill_fraction()
{

        return grass_hill_fraction;

}

float sine_grass_get_lawn_length_variation_frac()
{

        return lawn_length_variation_frac;

}

void sine_grass_set_grass_mowed(const int value)
{

        grass_mowed = value;

}

void sine_grass_set_grass_hill_fraction(const float value)
{

        grass_hill_fraction = value;

}

void sine_grass_set_lawn_length_variation_frac(const float value)
{
	fprintf(stdout, "LLVFb = %f\n", value);

        lawn_length_variation_frac = value;

}

// GETTERS/SETTERS for some of the members of the the_params struct
int sine_grass_params_lawn_length_variation_max_get()
{

	return the_params->lawn_length_variation_max;

}
 
int sine_grass_params_lawn_length_get()
{

	return the_params->lawn_length;

}
 
float sine_grass_params_sine_grass_freq_get()
{

	return the_params->sine_grass_freq;

}
 

void sine_grass_params_lawn_length_variation_max_set(const int val)
{

	the_params->lawn_length_variation_max = val;

}
 
void sine_grass_params_lawn_length_set(const int val)
{

	the_params->lawn_length = val;

}
 
void sine_grass_params_sine_grass_freq_set(const float val)
{

	the_params->sine_grass_freq = val;

}

void sine_grass_set_self_control(const int value)
{
    self_control = value;
}
int sine_grass_get_self_control()
{
    return self_control;
}

// www.yfx.de
// www.nessunastoriabuongiornopolizia.de
